/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine HTTP Handler
//
// Douglas Thrift
//
// $Id: HttpHandler.h,v 1.22 2004/03/17 08:23:49 douglas Exp $

#ifndef _HttpHandler_h_
#define _HttpHandler_h_

#include "Search.h"
#include "URL.h"

// Lovely C Sockets!
#ifdef _WIN32
// Windows Sockets
#include <winsock2.h>
#include <windows.h>

inline int getpid() { return GetCurrentProcessId(); }
#else
// BSD Sockets
typedef int SOCKET;

#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#endif // _WIN32

#ifdef _OpenSSL_
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#endif

class HttpHandler
{
private:
	enum code {ok = 200, choices = 300, moved = 301, found = 302, notfound =
		404, internal = 500};
#ifdef _WIN32
	WSADATA data;
#endif // _WIN32
	SOCKET http;
#ifdef _OpenSSL_
	SSL_CTX* ctx;
	SSL* ssl;
	bool tls;
#endif
	char* buffer;
	bool binary;
	stringstream page;
	string type;
	unsigned length;
	string location;
	bool chunked;
	void populate();
	void putline(const string line = "");
	string getline();
	void error(const string& prefix, bool host = false);
#ifdef _OpenSSL_
	void error(const string& prefix, int number);
	bool starttls();
#endif
// friends:
	friend istream& operator>>(istream& is, code& data) { int number; is
		>> number; data = code(number); return is; }
public:
	HttpHandler();
	~HttpHandler();
	bool handle(URL& url, const string referer = "", bool head = false);
	iostream& pageStream() { return page; }
	istream& getline(string& line) { return std::getline(page, line); }
	istream& getline(string& line, char end) { return std::getline(page, line,
		end); }
	bool good() { return page.good(); }
	void clear();
	string contentType() { return type; }
	unsigned contentLength() { return length; }
	string redirect() { return location; }
};

#endif // _HttpHandler_h_
