/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine Ranker
//
// Douglas Thrift
//
// $Id: Ranker.h,v 1.5 2004/01/01 23:00:34 douglas Exp $

#ifndef _Ranker_h_
#define _Ranker_h_

#include "Search.h"
#include "Page.h"

class Ranker : public Page
{
private:
	enum state { all, title, url, text };
	string sample;
	map<unsigned, unsigned> occurrencesTitle;
	map<unsigned, unsigned> occurrencesDescription;
	map<unsigned, unsigned> occurrencesText;
	unsigned value;
	vector<string> required;
	vector<string> excluded;
	vector<string> eitherOr;
	unsigned requiredValue;
	unsigned excludedValue;
	unsigned eitherOrValue;
	string site;
	state allIn;
	string lowerURL;
	string lowerAddress;
	string lowerTitle;
	string lowerText;
	void rank();
	void checkRequired();
	void checkExcluded();
	void checkEitherOr();
	unsigned find(string word, const string& where);
	unsigned find(string word, const string& where, map<unsigned, unsigned>&
		occurrences);
	unsigned phrase(const string& phrase, const string& where);
	unsigned phrase(const string& phrase, const string& where, map<unsigned,
		unsigned>& occurrences);
	unsigned phrase(const vector<string>& words, unsigned word, unsigned&
		begin, bool start, const string& where);
	unsigned phrase(const vector<string>& words, unsigned word, unsigned&
		begin, bool start, const string& where, map<unsigned, unsigned>&
		occurrences);
	unsigned evaluate(vector<unsigned>& ins);
	void decrap(string& crap);
public:
	Ranker();
	Ranker(Page& page);
	~Ranker() {}
	void rank(vector<string> query);
	void setSample();
	string getTitle();
	string getDescription();
	string getSample() { return sample; }
	bool operator==(const unsigned number) const;
	bool operator==(const Ranker& ranker) const;
	bool operator!=(const unsigned number) const;
	bool operator!=(const Ranker& ranker) const;
	bool operator<(const unsigned number) const;
	bool operator<(const Ranker& ranker) const;
	bool operator>(const unsigned number) const;
	bool operator>(const Ranker& ranker) const;
	bool operator<=(const unsigned number) const
	{
		return *this == number || *this < number;
	}
	bool operator<=(const Ranker& ranker) const
	{
		return *this == ranker || *this < ranker;
	}
	bool operator>=(const unsigned number) const
	{
		return *this == number || *this > number;
	}
	bool operator>=(const Ranker& ranker) const
	{
		return *this == ranker || *this > ranker;
	}
// friends:
	friend bool operator==(const unsigned number, const Ranker& ranker)
	{
		return ranker == number;
	}
	friend bool operator!=(const unsigned number, const Ranker& ranker)
	{
		return ranker != number;
	}
	friend bool operator>(const unsigned number, const Ranker& ranker)
	{
		return ranker < number;
	}
	friend bool operator<(const unsigned number, const Ranker& ranker)
	{
		return ranker > number;
	}
	friend bool operator>=(const unsigned number, const Ranker& ranker)
	{
		return ranker <= number;
	}
	friend bool operator<=(const unsigned number, const Ranker& ranker)
	{
		return ranker >= number;
	}
};

#endif // _Ranker_h_
