/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine Searcher
//
// Douglas Thrift
//
// $Id: Searcher.h,v 1.4 2004/01/01 23:00:34 douglas Exp $

#ifndef _Searcher_h_
#define _Searcher_h_

#include "Search.h"
#include "Page.h"
#include "URL.h"
#include "Ranker.h"

typedef multiset<Ranker, greater<Ranker> > MultiSet;
typedef multiset<Ranker, greater<Ranker> >::iterator MultiSetIterator;

class Searcher
{
private:
	string queryString;
	vector<string> query;
	MultiSet pages;
	set<string> common;
	vector<string> commonUsed;
	bool and_;
	bool or_;
	string ignore;
	clock_t start;
	clock_t finish;
	void search(Page& page);
	void setQuery();
	void setCommon();
	void setAnd(bool answer) { and_ = answer; }
	void setOr(bool answer) { or_ = answer; }
	void setIgnore(string& ignore) { this->ignore = ignore; }
public:
	Searcher(string& queryString);
	~Searcher() {}
	void search(vector<string> indices);
	vector<string> getQuery() { return query; }
	vector<string> getCommonUsed() { return commonUsed; }
	bool getAnd() { return and_; }
	bool getOr() { return or_; }
	string getIgnore() { return ignore; }
	string getQueryString() { return queryString; }
	MultiSet& getPages() { return pages; }
	double time() { return double(finish - start) / double(CLOCKS_PER_SEC); }
};

#endif // _Searcher_h_
