/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine URL
//
// Douglas Thrift
//
// $Id: URL.h,v 1.6 2004/01/01 23:00:34 douglas Exp $

#ifndef _URL_h_
#define _URL_h_

#include "Search.h"

class URL
{
private:
	string address;
	unsigned port;
	string path;
#ifdef _OpenSSL_
	bool tls;
#endif
public:
#ifndef _OpenSSL_
	URL() { port = 80; }
#else
	URL() { port = 80; tls = false; }
#endif
	URL(const string& url);
	URL(const string& address, unsigned port, const string& path);
#ifdef _OpenSSL_
	URL(const string& address, unsigned port, const string& path, bool tls);
#endif
	virtual ~URL() {}
	string getURL();
	string getAddress() { return address; }
	unsigned getPort() { return port; }
	string getPath() { return path; }
#ifdef _OpenSSL_
	bool getTls() { return tls; }
#endif
	void setURL(const URL& url);
	void setURL(const string& url);
	void setAddress(const string& address) { this->address = address; }
	void setPort(unsigned port) { this->port = port; }
	void setPath(const string& path);
#ifdef _OpenSSL_
	void setTls(bool tls) { this->tls = tls; }
#endif
	bool empty() { return address == "" || path == ""; }
// friends:
	friend ostream& operator<<(ostream& os, URL& data);
};

string getLink(string link, URL& url);

#endif // _URL_h_
